/* jshint esversion : 6 */


const IDBOpenRequest = require('./IDB/IDBOpenRequest');
const IDBCloseConnection = require('./IDB/IDBCloseConnection');
//
const IDBTransaction = require('./IDB/IDBTransaction');
const IDBCheckStoreConfig = require('./IDB/IDBCheckStoreConfig');
const IDBObjectStoreNames = require('./IDB/IDBObjectStoreNames');
const IDBObjectStore = require('./IDBObjectStores/IDBObjectStore');
//
const { useWorker } = require('app-utilities');
const IDBCron = require('./IDBCron');


module.exports = dbName => {

    class IDB extends IDBCron {
        constructor(dbName, alias) {
            super();
            this.dbName = dbName;
            this.alias = alias || dbName;
            this.db = null;
            this._ready = $q.defer();
            this._stores = {};
        }

        get ready() {
            return this._ready.promise;
        }

        set ready(o) {
            this._ready[o === 'y' ? 'resolve' : 'reject']();
        }

        /* jshint ignore : start */
        async syncConfig(httpParams) {
            const { objectStores } = await this.xhr(httpParams).catch(e => { throw e; });

            return this.createObjectStores(objectStores);
        }

        /* jshint ignore : end */


        /**
         * open the connection defining the version of idb
         * @param {Number} [version] - change version
         * @param {Function} upgradeCallback - onupgraneeded
         * @returns {Promise} promise
         */
        connect({ version, onsuccess, onupgradeneeded, onerror, onabort, onclose }) {
            return IDBOpenRequest({ dbName: this.dbName, version, onsuccess, onupgradeneeded, onerror, onabort, onclose });
        }

        close(idb) {
            return IDBCloseConnection(idb);
        }

        /* jshint ignore : start */
        async transaction(storeNames, flag) {
            return new Promise((resolve, reject) => {
                this.connect({
                    onsuccess: async idb => {
                        const t = await IDBTransaction({ idb, storeNames, flag });
                        resolve(t);
                    },
                    onerror: e => { throw e; },
                    onabort: reject,
                    onclose: reject
                });
            });
        }

        /* jshint ignore : end */

        checkStoreConfig(storesConfig) {
            const req = { storesConfig, dbName: this.dbName };
            return (useWorker ? this.wq({ path: 'IDBCheckStoreConfig.js', req, shared: true }) : IDBCheckStoreConfig(req));
        }


        objectStoreNames() {
            const req = { dbName: this.dbName };
            return (useWorker ? this.wq({ path: 'IDBObjectStoreNames.js', req, shared: true }) : IDBObjectStoreNames(req));
        }


        /* jshint ignore: start */
        createObjectStores(a) {

            return new Promise(async resolve => {

                if (!Array.isArray(a)) a = [a];

                if (!a.length) return resolve();

                const opt = a.map(o => {
                    const { storeName, autoIncrement, keyPath, indexes } = this._stores[o.storeName] = IDBObjectStore(this, o);
                    this._stores[storeName].syncData(o.syncData);
                    return { storeName, autoIncrement, keyPath, indexes };
                });

                const { exec, configs } = await this.checkStoreConfig(opt);

                const diff = Object.keys(configs);


                if (exec) {
                    const x = Object.values(configs).map(o => o.storeName);
                    const first = x.shift();

                    const fn = storeName => {
                        const objectStore = this._stores[storeName];
                        return objectStore.init(diff);
                    };

                    await x.reduce(async(curr, next) => {
                        await curr;
                        return fn(next);
                    }, fn(first));

                    resolve(1);
                } else
                    resolve(0);
            });
        }


        async syncAllData(opt, callback) {
            await this.ready;
            opt.objectStoreNames = await this.objectStoreNames();
            return this.syncData(opt, callback);
        }


        async syncData(o, callback) {
            const { objectStoreNames } = o;
            //
            //
            const mapFn = e => {
                const objectStores = this._stores[e];
                return objectStores.syncData(o, callback).complete;
            };
            const queue = objectStoreNames.map(mapFn);

            return Promise.all(queue);
        }

        /* jshint ignore: end */


        /**
         * retriev object store 
         * @param {String} storeName - name of store
         * @returns {Object} objectStore
         */
        objectStore(storeName) {
            if (!this._stores[storeName])
                throw new Error('objectStore ' + storeName + ' not exists');

            return this._stores[storeName];
        }

    }

    return new IDB(dbName);

};