/* jshint esversion : 6 */
/* jshint ignore : start */

const {
    Insert,
    Get,
    GetAll,
    Count,
    Delete,
    Cursor,
    CursorDelete,
    CursorUpdate,
    BulkInsert: BulkUpdate
} = require('../queries/index');

const {
    useWorker,
    isUndefined
} = require('app-utilities');


module.exports = IDBObjectStore => {

    class IDBObjectStoreQueries extends IDBObjectStore {
        constructor(storeName, opt) {
            super(storeName, opt);
        }


        /* INSERT DATA */
        __insert(data, key, cmd, shared) {
            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                cmd,
                data,
                key
            };
            const path = 'queries/__Insert.js';

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, shared }) : Insert(req));
        }

        add(data, key, shared) {
            return this.__insert(data, key, 'add', shared);
        }

        put(data, key, shared) {
            return this.__insert(data, key, 'put', shared);
        }

        /* RETRIEVE ANY DATA */
        __get(key, cmd, shared) {
            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                cmd,
                key
            };
            const path = 'queries/__Get.js';

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, shared }) : Get(req));
        }

        get(key, shared) {
            return this.__get(key, 'get', shared).catch(e => { throw e; });
        }

        getKey(key, shared) {
            return this.__get(key, 'getKey', shared).catch(e => { throw e; });
        }


        /* RETRIEVE ALL DATA */
        __getAll(key, count, cmd, shared) {
            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                cmd,
                key,
                count
            };
            const path = 'queries/__GetAll.js';

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, shared }) : GetAll(req));
        }

        getAll(key, count, shared) {
            return this.__getAll(key, count, 'getAll', shared).catch(e => { throw e; });
        }

        getAllKeys(key, count, shared) {
            return this.__getAll(key, count, 'getAllKeys', shared).catch(e => { throw e; });
        }

        getModel() {
            return new Promise(async resolve => {


                if (this.onlineFirst) {
                    try {
                        this.syncWithServer({
                            onHttpData(m) { return resolve(m) }
                        });
                    } catch (err) {
                        const m = await this.getAll();
                        resolve(m);
                    }
                } else {
                    const m = await this.getAll();
                    if (!m || !m.length) {
                        this.syncWithServer({
                            onHttpData(m) { return resolve(m) }
                        });
                    } else resolve(m);
                }

            });
        }

        /* DELETE ANY DATA OR CLEAR ALL */
        delete(key, shared) {
            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                key
            };
            const path = 'queries/__Delete.js';

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, shared }) : Delete(req)).catch(e => { throw e; });;
        }

        /* COUNT DATA */
        count(key, shared) {
            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                key
            };
            const path = 'queries/__Count.js';

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, shared }) : Count(req)).catch(e => { throw e; });;
        }

        /* MASSIVE UPDATE */
        bulkUpdate(data, o, shared) {

            if (!o) o = {};
            const { key, direction } = o;

            const req = {
                dbName: this.dbName,
                storeNames: this.storeName,
                data,
                key,
                direction
            };
            const path = 'queries/__BulkInsert.js';
            const notify = true;

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, notify, shared: false }) : BulkUpdate(req)).catch(e => { throw e; });;
        }

        /* OPEN CURSOR */
        cursor(...args) {
            let [req, callBack] = setCursorParams(args);

            if (!req) req = {};
            req.dbName = this.dbName;
            req.storeNames = this.storeName;
            req.query = { cmd: 'sync' };

            return Cursor(req, callBack).catch(e => { throw e; });
        }


        __cursorQuery(req, callBack, shared, p) {

            if (!req) req = {};

            const syncScript = p === '__CursorUpdate.js' ? CursorUpdate : CursorDelete;

            req.dbName = this.dbName;
            req.storeNames = this.storeName;


            const path = `queries/${p}`;
            const notify = true;

            if (isUndefined(shared)) shared = true;
            return (useWorker ? this.wq({ path, req, notify, shared }) : syncScript(req, callBack));
        }

        cursorUpdate(req, callBack) {
            return this.__cursorQuery(req, callBack, req.shared, '__CursorUpdate.js').catch(e => { throw e; });
        }

        cursorDelete(req, callBack) {
            return this.__cursorQuery(req, callBack, req.shared, '__CursorDelete.js').catch(e => { throw e; });
        }

    }

    function setCursorParams(args) {
        const params = [];

        Array.from(args).forEach(v => {
            if (typeof v === "object")
                params[0] = v;
            else if (typeof v === "function")
                params[1] = v;
        });

        return params;
    }

    return IDBObjectStoreQueries;
};