/* jshint esversion : 6 */

const { searchCondition } = require('app-utilities');


module.exports = function({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction }) {
    /* jshint ignore : start */

    return function Cursor({ dbName, storeNames, key, direction, query = {} }, notify) {

        if (!notify) notify = () => {};

        return new Promise((resolve, reject) => {

            IDBOpenRequest({
                dbName,
                onsuccess: ({ idb }) => {

                    const catchFn = e => {
                        IDBCloseConnection(idb);
                        throw e;
                    };

                    let k;
                    if (isKeyRange(key)) k = IDBKeyRange(key.limit, key.range);

                    try {
                        const tx = IDBTransaction({ idb, storeNames, flag: "readwrite" });

                        const storeTx = (key && key.onIndex) ? tx.objectStore(storeNames).index(key.onIndex) : tx.objectStore(storeNames);
                        const cursor = k ? (direction ? storeTx.openCursor(k, direction) : storeTx.openCursor(k)) : storeTx.openCursor();


                        cursor.onsuccess = ev => {
                            const cursor = ev.target.result;
                            if (cursor) {

                                const { value, direction, key, primaryKey, state = "progress", source: { keyPath } } = cursor;

                                /**
                                 * query.where implementa query.searchCondition
                                 */
                                if (query.where) query.searchCondition = searchCondition(value, query.where);


                                switch (query.cmd) {

                                    case 'update':
                                        __CursorUpdate(cursor, query).catch(catchFn).then(notify);
                                        break;

                                    case 'delete':
                                        __CursorDelete(cursor, query).catch(catchFn).then(notify);
                                        break;


                                    case 'sync':
                                        if (notify) notify(cursor);
                                        break;

                                    default:
                                        if (notify) notify({ value, direction, key, primaryKey, state });
                                        break;
                                }


                                cursor.continue();
                            } else
                                notify({ state: 'end' });
                        };

                        tx.oncomplete = ev => {
                            IDBCloseConnection(idb);
                            resolve({ state: 'complete' });
                        };

                        tx.onerror = catchFn;

                        tx.onabort = catchFn;


                    } catch (e) {
                        catchFn(e);
                    }


                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });
        });
    };


    /* jshint ignore : end */
};



const getKeyPathValue = (o, kp) => JSON.stringify(Object.values(JSON.parse(JSON.stringify(o, kp))));
//

function __CursorUpdate(cursor, { data: newValue, searchCondition, onKeyUpdate }) {

    const { value: cursorValue, direction, key, primaryKey, source: { keyPath } } = cursor;

    const oldKeyValues = getKeyPathValue(cursorValue, keyPath);
    const newKeyValues = getKeyPathValue(newValue, keyPath);
    const updateByKeyValues = onKeyUpdate && (oldKeyValues === newKeyValues);

    return new Promise(resolve => {

        if (searchCondition || updateByKeyValues) {

            const req = cursor.update(newValue);
            req.onsuccess = () => {
                resolve({
                    oldValue: cursorValue,
                    newValue,
                    direction,
                    key,
                    primaryKey,
                    state: "update",
                    msg: "progress"
                });
            };
            req.onerror = () => {
                __CursorDelete(cursor, { searchCondition, onKeyUpdate })
                    .catch(e => { throw e; })
                    .then(resolve);
            };
        }
    });
}



function __CursorDelete(cursor, { data: newValue, searchCondition, onKeyUpdate }) {

    const { value: cursorValue, direction, key, primaryKey, source: { keyPath } } = cursor;

    let oldKeyValues, newKeyValues, updateByKeyValues;

    if (newValue && onKeyUpdate) {
        oldKeyValues = getKeyPathValue(cursorValue, keyPath);
        newKeyValues = getKeyPathValue(newValue, keyPath);
        updateByKeyValues = onKeyUpdate && (oldKeyValues === newKeyValues);
    }

    return new Promise(resolve => {

        if (searchCondition || updateByKeyValues) {
            const req = cursor.delete();
            req.onsuccess = () => {
                resolve({
                    deletedValue: cursorValue,
                    direction,
                    key,
                    primaryKey,
                    state: "delete",
                    msg: "progress"
                });
            };
            req.onerror = ev => { throw ev; };
        }
    });
}